#include <Application.h>
#include <Bitmap.h>
#include <Mime.h>
#include <Alert.h>
#include <Messenger.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <OS.h>
#include <Alert.h>
#include "FtpPositive.h"
#include "FTPWindow.h"
#include "TCPStream.h"



// ----------------------------------- TLogView -------------------------------------

TLogView::TLogView(BRect rect, const char *name, BRect textRect)
	:	BTextView(rect, name, textRect, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_BOTTOM, B_WILL_DRAW)
{
	this->MakeEditable(false);
	this->SetWordWrap(true);
	this->SetStylable(true);
}

TLogView::~TLogView()
{
}

void TLogView::FrameResized(float width, float height)
{
	BRect rect(this->TextRect());
	rect.right = width;
	this->SetTextRect(rect);
	this->BTextView::FrameResized(width, height);
}

void TLogView::InsertText(const char *text, int32 length, int32 offset, const text_run_array *runs)
{
	BString str(text, length);
	str.ReplaceAll("\r\n", "\n");
	BTextView::InsertText(str.String(), str.Length(), offset, runs);
}

TLogView& TLogView::operator<<(const char *string)
{
	int32 len = this->TextLength();
	this->Select(len, len);
	this->Insert(string);
	this->ScrollToSelection();
	return *this;
}

TLogView& TLogView::operator<<(int val)
{
	BString s;
	s << val;
	return *this << s.String();
}


// ----------------------------------- TFtpLooper -----------------------------------

TFtpLooper::TFtpLooper(BLooper *owner, const connect_param_t *param)
	:	BLooper("FtpLooper")
{
	fOwner = owner;
	fPollingTimer = new BMessageRunner(this, new BMessage(MSG_POLLING_TIMER), 500000);
	fMessenger = new BMessenger(owner);
	fFtpClient = new TFTPClient();
	fHost.SetTo(param->host);
	fPort = param->port;
	fUsername.SetTo(param->username);
	fPassword.SetTo(param->password);
}

TFtpLooper::~TFtpLooper()
{
	if (fFtpClient) delete fFtpClient;
	if (fPollingTimer) delete fPollingTimer;
}

TFtpLooper& TFtpLooper::operator<<(const char *string)
{
	BMessage msg(MSG_LOG);
	msg.AddString("log", string);
	fMessenger->SendMessage(msg);
	return *this;
}

TFtpLooper& TFtpLooper::operator<<(BString string)
{
	return *this << string.String();
}

TFtpLooper& TFtpLooper::operator<<(int val)
{
	BString s;
	s << val;
	return *this << s.String();
}

void TFtpLooper::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case MSG_FTP_CONNECT:	Connect();	break;
		case MSG_FTP_PASV_LIST:	PasvList();	break;
		case MSG_FTP_CHDIR: {
			BString dir;
			if (msg->FindString("dir", &dir) == B_OK) this->Chdir(dir.String());
			break;
		}
		case MSG_POLLING_TIMER: {
			if (fFtpClient->IsReceivable(0, 0)) {
				if (fFtpClient->Readres(1000000) != B_OK) {
					*this << "Error: " << fFtpClient->StrError() << "\n";
					delete fFtpClient;
					fFtpClient = NULL;
					delete fPollingTimer;
					fPollingTimer = NULL;
					fMessenger->SendMessage(MSG_DISCONNECTED);
					return;
				}
				*this << fFtpClient->StrReply();
			}
			break;
		}
		default: BLooper::MessageReceived(msg);
	}
}

void TFtpLooper::Abort()
{
	if (fFtpClient) fFtpClient->Abort();
}

void TFtpLooper::Connect()
{
	uint32 r;
	*this << "Connecting to " << fHost << " (" << (int)fPort << ")\n";
	
	// 接続
	if (fFtpClient->Connect(fHost.String(), fPort) != B_OK) {
		*this << "Error: " << fFtpClient->StrError() << "\n";
		goto _err_exit;
	}
	*this << fFtpClient->StrReply();
	r =fFtpClient->Reply();
	if ((r < 200) || (r > 299)) goto _err_exit;
	
	// ID 送り
	*this << ">USER " << fUsername << "\n";
	if (fFtpClient->USER(fUsername.String()) != B_OK) {
		*this << "Error: " << fFtpClient->StrError() << "\n";
		goto _err_exit;
	}
	*this << fFtpClient->StrReply();
	r =fFtpClient->Reply();
	if ((r < 300) || (r > 399)) goto _err_exit;
	
	//パスワード送り
	*this << ">PASS **********\n";
	if (fFtpClient->PASS(fPassword.String()) != B_OK) {
		*this << "Error: " << fFtpClient->StrError() << "\n";
		goto _err_exit;
	}
	*this << fFtpClient->StrReply();
	r =fFtpClient->Reply();
	if ((r < 200) || (r > 299)) goto _err_exit;
	
	fMessenger->SendMessage(MSG_FTP_OK);
	return;
_err_exit:
	
	fMessenger->SendMessage(MSG_FTP_FAILED);
	return;
}

void TFtpLooper::PasvList()
{
	uint32 r;
	struct sockaddr_in addr;
	BString xpwd_result, lines;
	BMessage msg;
	BMallocIO mallocIO;
	TTCPStream *tcpStream;
	
	// PASV モード
	*this << ">PASV\n";
	if (fFtpClient->PASV(&addr) != B_OK) {
		*this << "Error: " << fFtpClient->StrError();
		goto _err_exit;
	}
	*this << fFtpClient->StrReply();
	r =fFtpClient->Reply();
	if ((r < 200) || (r > 299)) goto _err_exit;
	
	// NLST
	*this << ">NLST -alL\n";
	if (fFtpClient->NLST(&addr, true) != B_OK) {
		*this << "Error: " << fFtpClient->StrError();
		goto _err_exit;
	}
	
	// データ受信
	tcpStream = new TTCPStream(&mallocIO, &addr, 10000000);
	
	fFtpClient->Readres(10000000);
	*this << fFtpClient->StrReply();
	if (r < 150) {
		// データ受信失敗
		tcpStream->Abort();
		delete tcpStream;
		goto _err_exit;
	}
	while (tcpStream->Result() == EINPROGRESS) snooze(10000);
	if (tcpStream->Result() == B_OK) {
		lines.SetTo((char *)mallocIO.Buffer(), mallocIO.BufferLength());
		//fprintf(stderr, "%s\n", lines.String());
	} else {
		//
		fprintf(stderr, "ERROR\n");
	}
	delete tcpStream;
	
	fFtpClient->Readres(10000000);
	*this << fFtpClient->StrReply();
	
	((char *)mallocIO.Buffer())[mallocIO.BufferLength()] = 0;
	
	// カレントディレクトリの確認
	*this << ">XPWD\n";
	if (fFtpClient->XPWD(&xpwd_result) != B_OK) {
		*this << "Error: " << fFtpClient->StrError();
		goto _err_exit;
	}
	*this << fFtpClient->StrReply();
	r =fFtpClient->Reply();
	if ((r < 200) || (r > 299)) goto _err_exit;
	
	msg.what = MSG_WALKED;
	msg.AddString("xpwd", xpwd_result);
	msg.AddString("dirent", (char *)mallocIO.Buffer());
	fMessenger->SendMessage(msg);
	fMessenger->SendMessage(MSG_FTP_OK);
	
	return;
	
_err_exit:
	
	fMessenger->SendMessage(MSG_FTP_FAILED);
	return;
}

void TFtpLooper::Chdir(const char *dir)
{
	uint32 r;
	
	// CWD
	*this << ">CWD " << dir << "\n";
	if (fFtpClient->CWD(dir) != B_OK) {
		*this << "Error: " << fFtpClient->StrError();
		goto _err_exit;
	}
	*this << fFtpClient->StrReply();
	r =fFtpClient->Reply();
	if ((r < 200) || (r > 299)) goto _err_exit;
	
	fMessenger->SendMessage(MSG_FTP_OK);
	
	return;
	
_err_exit:
	
	fMessenger->SendMessage(MSG_FTP_FAILED);
	return;
}


// ----------------------------------- TFTPWindow -------------------------------------

TFTPWindow::TFTPWindow(BRect frame, const char *name)
	:	BWindow(frame, name, B_DOCUMENT_WINDOW, 0)
{
	fStage = STAGE_SUSPENDED;
	
	fFtpLooper = NULL;
	fBookmarkConfig = NULL;
	fDirentParser = NULL;
	
	float minW, minH, maxW, maxH;
	this->GetSizeLimits(&minW, &maxW, &minH, &maxH);
	this->SetSizeLimits(280, maxW, 250, maxH);
	
	BRect rect;
	
	BView *bgview = new BView(Bounds(), "BGView",
		B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS);
	bgview->SetViewColor(217, 217, 217);
	this->AddChild(bgview);
	
	// Main Menu
	BMenuBar *mainMenu = new BMenuBar(BRect(), "MainMenuBar");
	bgview->AddChild(mainMenu);
	
	// Remote Path View
	rect.left = 10;
	rect.right = bgview->Bounds().right - 25;
	rect.top = mainMenu->Frame().bottom + 7;
	rect.bottom = rect.top + 12;
	fRemotePathView = new BTextControl(rect, "RemotePathView", "Remote Path :", "",
		new BMessage(MSG_REMOTE_PATH_CHANGED), B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
	fRemotePathView->SetDivider(90);
	bgview->AddChild(fRemotePathView);
	
	// Cancel Button
	rect.left = bgview->Bounds().right - 20;
	rect.right = rect.left + 17;
	rect.top = mainMenu->Frame().bottom + 7;
	rect.bottom = rect.top + 20;
	BButton *cancelButton = new BButton(rect, "CancelButton", "X",
		new BMessage(MSG_CANCEL), B_FOLLOW_RIGHT);
	bgview->AddChild(cancelButton);
	cancelButton->ResizeTo(17, 18);
	
	// Remote File View
	BStringColumn *nameColumn = new BStringColumn("Name", 150, 0, INT_MAX, 0);
	BStringColumn *intNameColumn = new BStringColumn("Internal Name", 150, 0, INT_MAX, 0);
	rect.left = 5;
	rect.right = bgview->Bounds().right - 5;
	rect.top = fRemotePathView->Frame().bottom + 7;
	rect.bottom = bgview->Bounds().bottom - 107;
	fRemoteFileView = new BColumnListView(rect, "RemoteFileView",
		B_FOLLOW_ALL_SIDES, B_NAVIGABLE, B_FANCY_BORDER);
	fRemoteFileView->AddColumn(new BBitmapColumn("", 28, 28, 28), CLM_ICON);
	fRemoteFileView->AddColumn(nameColumn, CLM_NAME);
	fRemoteFileView->AddColumn(intNameColumn, CLM_INTERNAL_NAME);
	fRemoteFileView->AddColumn(new BSizeColumn("Size", 90, 0, INT_MAX, B_ALIGN_RIGHT), CLM_SIZE);
	fRemoteFileView->AddColumn(new BStringColumn("Date", 100, 0, INT_MAX, 0), CLM_DATE);
	fRemoteFileView->AddColumn(new BStringColumn("Permission", 80, 0, INT_MAX, 0), CLM_PERMISSION);
	fRemoteFileView->AddColumn(new BStringColumn("Owner", 70, 0, INT_MAX, 0), CLM_OWNER);
	fRemoteFileView->AddColumn(new BStringColumn("Group", 70, 0, INT_MAX, 0), CLM_GROUP);
	fRemoteFileView->SetTarget(this);
	fRemoteFileView->SetInvocationMessage(new BMessage(MSG_ENTRY_DOUBLE_CLICKED));
	bgview->AddChild(fRemoteFileView);
	intNameColumn->SetVisible(false);
	fRemoteFileView->SetSortingEnabled(true);
	fRemoteFileView->SetSortColumn(nameColumn, false, true);
	
	// Log View
	rect.left = 5;
	rect.right = bgview->Bounds().right - B_V_SCROLL_BAR_WIDTH - 5;
	rect.top = bgview->Bounds().bottom - 95;
	rect.bottom = bgview->Bounds().bottom - 20;
	fLogView = new TLogView(rect, "LogView", BRect(2, 2, rect.Width(), rect.Height()));
	BScrollView *logScrollView = new BScrollView("logScrollView", fLogView,
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_BOTTOM, B_WILL_DRAW, false, true);
	bgview->AddChild(logScrollView);
	
	// Status View
	rect.left = 5;
	rect.right = bgview->Bounds().right - B_V_SCROLL_BAR_WIDTH - 5;
	rect.top = logScrollView->Frame().bottom + 1;
	rect.bottom = bgview->Bounds().bottom;
	fStatusView = new BStringView(rect, "StatusView", "0 items", B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	bgview->AddChild(fStatusView);
	
	// File Menu
	BMenu *fileMenu = new BMenu("File");
	mainMenu->AddItem(fileMenu);
	fileMenu->AddItem(new BMenuItem("Upload", NULL));
	fileMenu->AddItem(new BMenuItem("Download", NULL));
	fileMenu->AddSeparatorItem();
	fileMenu->AddItem(new BMenuItem("About FtpPositive", new BMessage(B_ABOUT_REQUESTED)));
	fileMenu->AddSeparatorItem();
	fileMenu->AddItem(new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED)));
	
	// Connect Menu
	fConnectMenu = new BMenu("Connect");
	mainMenu->AddItem(fConnectMenu);
	fConnectMenu->AddItem(new BMenuItem("Show Bookmarks", new BMessage(MSG_SHOW_BOOKMARK)));
	fConnectMenu->AddSeparatorItem();
	
	BDirectory dir(BOOKMARKS_DIR);
	LoadBookmarks(&dir, fConnectMenu);
	
	// Encoding Menu
	fEncodingMenu = new BMenu("Encoding");
	mainMenu->AddItem(fEncodingMenu);
	fEncodingMenu->SetRadioMode(true);
	fNoEncoder = new TEncoderMenuItem("None", NULL);
	fEncodingMenu->AddItem(fNoEncoder);
	fEncodingMenu->AddSeparatorItem();
	fNoEncoder->SetMarked(true);
	for (int32 i = 0; i < app_encoderlist->CountItems(); i++) {
		encoder_addon_t *encoder = (encoder_addon_t *)app_encoderlist->ItemAt(i);
		fEncodingMenu->AddItem(new TEncoderMenuItem(encoder->name, encoder));
	}
	
	
	// 
	*fLogView << "Welcome to FtpPositive Version " VERSION "\n" COPY "\n";
	fRemoteFileView->MakeFocus(true);
}

TFTPWindow::~TFTPWindow()
{
	BString s;
	s.SetTo(""); s << (int)Frame().left;   app_config->Write("frame_left",   s.String());
	s.SetTo(""); s << (int)Frame().top;    app_config->Write("frame_top",    s.String());
	s.SetTo(""); s << (int)Frame().right;  app_config->Write("frame_right",  s.String());
	s.SetTo(""); s << (int)Frame().bottom; app_config->Write("frame_bottom", s.String());
	Clear();
}

void TFTPWindow::Clear()
{
	if (fStage != STAGE_SUSPENDED) Disconnect();
	
	if (fBookmarkConfig) {
		fBookmarkConfig->Update();
		delete fBookmarkConfig;
		fBookmarkConfig = NULL;
	}
	if (fDirentParser) {
		delete fDirentParser;
		fDirentParser = NULL;
	}
	
	fRemoteFileView->Clear();
	fRemotePathView->SetText("");
	fStatusView->SetText("0 items");
	this->SetTitle("FtpPositive");
	
	fStage = STAGE_SUSPENDED;
}

bool TFTPWindow::QuitRequested()
{
	BAlert *alert = new BAlert("FtpPositive", "Really quit?",
							"Yes", "No", NULL,
							B_WIDTH_AS_USUAL, B_WARNING_ALERT);
	if (alert->Go() == 0) {
		be_app_messenger.SendMessage(B_QUIT_REQUESTED);
		return true;
	}
	return false;
}

void TFTPWindow::MessageReceived(BMessage *msg)
{
	status_t s;
	switch(msg->what) {
		case MSG_FTP_OK: {
			FtpOk();
			break;
		}
		case MSG_FTP_FAILED: {
			FtpFailed();
			break;
		}
		case MSG_WALKED: {
			BString xpwd_result, dirent;
			if (msg->FindString("xpwd", &xpwd_result) == B_OK) {
				fRemotePathView->SetText(xpwd_result.String());
			}
			if (msg->FindString("dirent", &dirent) == B_OK) {
				fDirentParser->MakeEmpty();
				if ((s = fDirentParser->AddEntries(dirent.String())) != B_OK) {
					*fLogView << "Error: " << strerror(s) << "\n";
					FtpFailed();
					break;
				}
				DirlistChanged();
			}
			break;
		}
		case MSG_SHOW_BOOKMARK: {
			ShowBookmark();
			break;
		}
		case MSG_BOOKMARK_SELECTED: {
			BString path;
			if (msg->FindString("entry", &path) != B_OK) return;
			BookmarkSelected(path.String());
			break;
		}
		case MSG_ENCODER_SELECTED: {
			EncoderChanged();
			break;
		}
		case MSG_REMOTE_PATH_CHANGED: {
			Chdir(fRemotePathView->Text());
			break;
		}
		case MSG_CANCEL: {
			if (fFtpLooper) fFtpLooper->Abort();
			if (fStage > STAGE_IDLE) *fLogView << "Aborted.\n";
			break;
		}
		case MSG_LOG: {
			BString str;
			if (msg->FindString("log", &str) == B_OK) *fLogView << str.String();
			break;
		}
		case MSG_ENTRY_DOUBLE_CLICKED: {
			RemoteFileDoubleClicked();
			break;
		}
		case MSG_DISCONNECTED: {
			Disconnect();
			break;
		}
		case B_ABOUT_REQUESTED: {
			(new BAlert("", "FtpPositive " VERSION "\n" COPY, "OK"))->Go();
			break;
		}
		default: BWindow::MessageReceived(msg);
	}
}

void TFTPWindow::AddRemoteFileItem(const char *name, uint32 size,
	const char *date, uint32 perm, const char *owner, const char *group)
{
	// ディレクトリ、またはファイルタイプのアイコンイメージを取得
	BMimeType mime;
	BBitmap *icon = new BBitmap(BRect(0, 0, B_MINI_ICON - 1, B_MINI_ICON - 1), B_CMAP8);
	if (perm & 01000) {
		mime.SetTo("application/x-vnd.Be-directory");
	} else {
		mime.SetTo("application/octet-stream");
		const char *ext = strrchr(name, '.');
		if (ext) {
			ext++;
			for (int32 i = 0; i < app_extlist->CountItems(); i++) {
				ext_item_t *item = (ext_item_t *)app_extlist->ItemAt(i);				
				if (strcasecmp(ext, item->ei_ext) == 0) {
					mime.SetTo(item->ei_mimetype);
					break;
				}
			}
		}
	}
	if (mime.GetIcon(icon, B_MINI_ICON) != B_OK) {
		mime.SetTo("application/octet-stream");
		mime.GetIcon(icon, B_MINI_ICON);
	}
	
	// パーミッション文字列変換
	BString strperm;
	strperm << ((perm & 01000) ? "d" : "-")
	 << ((perm & 0400) ? "r" : "-") << ((perm & 0200) ? "w" : "-") << ((perm & 0100) ? "x" : "-")
	 << ((perm & 0040) ? "r" : "-") << ((perm & 0020) ? "w" : "-") << ((perm & 0010) ? "x" : "-")
	 << ((perm & 0004) ? "r" : "-") << ((perm & 0002) ? "w" : "-") << ((perm & 0001) ? "x" : "-");
	
	// ファイル名文字コード変換
	BString utf8name(name);
	TEncoderMenuItem *encMenuItem = (TEncoderMenuItem *)fEncodingMenu->FindMarked();
	if (encMenuItem != NULL) {
		encoder_addon_t *encoderAddon = encMenuItem->EncoderAddon();
		if (encoderAddon != NULL) {
			encoderAddon->fp_text_convert(name, &utf8name, true);
		}
	}
	
	// リスト表示
	BRow *row = new BRow();
	row->SetField(new BBitmapField(icon), CLM_ICON);
	row->SetField(new BStringField(utf8name.String()), CLM_NAME);
	row->SetField(new BStringField(name), CLM_INTERNAL_NAME);
	row->SetField(new BSizeField(size), CLM_SIZE);
	row->SetField(new BStringField(date), CLM_DATE);
	row->SetField(new BStringField(strperm.String()), CLM_PERMISSION);
	row->SetField(new BStringField(owner), CLM_OWNER);
	row->SetField(new BStringField(group), CLM_GROUP);
	fRemoteFileView->AddRow(row);
}

void TFTPWindow::SetBusy(bool busy)
{
	fRemotePathView->SetEnabled(!busy);
	fConnectMenu->SetEnabled(!busy);
	fEncodingMenu->SetEnabled(!busy);
}

void TFTPWindow::FtpOk()
{
	switch(fStage) {
		case STAGE_SUSPENDED:
			break;
		case STAGE_CONNECT:
			PasvList();
			break;
		case STAGE_IDLE:
			break;
		case STAGE_PASVLIST:
			fStage = STAGE_IDLE;
			fRemotePathView->TextView()->SelectAll();
			break;
		case STAGE_CHDIR:
			PasvList();
			break;
	}
	
	if (fStage == STAGE_IDLE) SetBusy(false);
}

void TFTPWindow::FtpFailed()
{
	switch(fStage) {
		case STAGE_SUSPENDED:
			break;
		case STAGE_CONNECT:
			Clear();
			break;
		case STAGE_IDLE:
			break;
		case STAGE_PASVLIST:
			fStage = STAGE_IDLE;
			break;
		case STAGE_CHDIR:
			fStage = STAGE_IDLE;
			break;
	}
	
	if (fStage == STAGE_IDLE) SetBusy(false);
}

void TFTPWindow::ShowBookmark() const
{
	BEntry entry(BOOKMARKS_DIR);
	if (entry.InitCheck() != B_OK) return;
	
	BMessage msg(B_REFS_RECEIVED);
	entry_ref ref;
	if (entry.GetRef(&ref) != B_OK) return;
	
	msg.AddRef("refs", &ref);
	BMessenger("application/x-vnd.Be-TRAK").SendMessage(&msg);
}

status_t TFTPWindow::LoadBookmarks(BDirectory *dir, BMenu *menu)
{
	status_t e;
	BEntry entry;
	char leafName[B_FILE_NAME_LENGTH];
	if ((e = dir->InitCheck()) != B_OK) return e;
	
	while (dir->GetNextEntry(&entry) == B_OK) {
		entry.GetName(leafName);
		if (entry.IsDirectory()) {
			BDirectory ndir(dir, leafName);
			BMenu *subMenu = new BMenu(leafName);
			menu->AddItem(subMenu);
			LoadBookmarks(&ndir, subMenu);
		} else {
			AddBookmark(&entry, menu);
		}
	}
	
}

status_t TFTPWindow::AddBookmark(BEntry *entry, BMenu *menu)
{
	BPath path(entry);
	TConfigFile conf(path.Path());
	BString host;
	conf.Read("host", &host, "");
	if (host.Length() == 0) return B_ERROR;
	
	BMessage *msg = new BMessage(MSG_BOOKMARK_SELECTED);
	msg->AddString("entry", path.Path());
	BMenuItem *item = new BMenuItem(path.Leaf(), msg);
	menu->AddItem(item);
	
	return B_OK;
}

void TFTPWindow::EncoderChanged()
{
	TEncoderMenuItem *encMenuItem = (TEncoderMenuItem *)fEncodingMenu->FindMarked();
	
	for(int32 i = 0; i < fRemoteFileView->CountRows(); i++) {
		BRow *row = fRemoteFileView->RowAt(i);
		BStringField *nameField = (BStringField *)row->GetField(CLM_NAME);		
		BStringField *intNameField = (BStringField *)row->GetField(CLM_INTERNAL_NAME);	
			
		// ファイル名文字コード変換
		BString utf8name(intNameField->String());
		if (encMenuItem != NULL) {
			encoder_addon_t *encoderAddon = encMenuItem->EncoderAddon();
			if (encoderAddon != NULL) {
				encoderAddon->fp_text_convert(intNameField->String(), &utf8name, true);
			}
		}
		nameField->SetString(utf8name.String());
	}
	fRemoteFileView->Refresh();
}

void TFTPWindow::BookmarkSelected(const char *path)
{
	Clear();
	
	fBookmarkConfig = new TConfigFile(path);
	if (fBookmarkConfig->Status() != B_OK) {
		BString msg("Bookmark load error.\n");
		msg << strerror(fBookmarkConfig->Status());
		(new BAlert("", msg.String(), "OK"))->Go();
		delete fBookmarkConfig;
		fBookmarkConfig = NULL;
		return;
	}
	BString host, username, password, port, encoder;
	fBookmarkConfig->Read("host", &host, "");
	fBookmarkConfig->Read("username", &username, "");
	fBookmarkConfig->Read("password", &password, "");
	fBookmarkConfig->Read("port", &port, "21");
	fBookmarkConfig->Read("encoder", &encoder, "");
	
	fNoEncoder->SetMarked(true);
	if (encoder.Length() > 0) {
		TEncoderMenuItem *encItem = (TEncoderMenuItem *)fEncodingMenu->FindItem(encoder.String());
		if (encItem) encItem->SetMarked(true);
	}
	
	connect_param_t param;
	param.host = host.String();
	param.port = atoi(port.String());
	param.username = username.String();
	param.password = password.String();
	Connect(&param);
}

void TFTPWindow::Connect(const connect_param_t *param)
{
	status_t s;
	fStage = STAGE_CONNECT;
	
	if (fFtpLooper != NULL) {
		fFtpLooper->Abort();
		fFtpLooper->Lock();
		fFtpLooper->Quit();
		fFtpLooper = NULL;
	}
	
	fFtpLooper = new TFtpLooper(this, param);
	fFtpLooper->Run();
	if ((s = BMessenger(fFtpLooper).SendMessage(MSG_FTP_CONNECT)) != B_OK) {
		*fLogView << "Error: " << strerror(s) << "\n";
		return;
	}
	
	if (fDirentParser) delete fDirentParser;
	fDirentParser = new TGenericDirentParser();
	
	BString title(param->host);
	title << " - FtpPositive";
	this->SetTitle(title.String());
}

void TFTPWindow::PasvList()
{
	status_t s;
	if ((s = BMessenger(fFtpLooper).SendMessage(MSG_FTP_PASV_LIST)) != B_OK) {
		*fLogView << "Error: " << strerror(s) << "\n";
		return;
	}
	fStage = STAGE_PASVLIST;
	SetBusy(true);
}

void TFTPWindow::Disconnect()
{
	if (fFtpLooper) {
		fFtpLooper->Abort();
		fFtpLooper->Lock();
		fFtpLooper->Quit();
		fFtpLooper = NULL;
	}
	*fLogView << "Disconnected.\n";
	fStage = STAGE_SUSPENDED;
	SetBusy(false);
}

void TFTPWindow::Chdir(const char *dir)
{
	status_t s;
	BMessage msg(MSG_FTP_CHDIR);
	if ((s = msg.AddString("dir", dir)) != B_OK) {
		*fLogView << "Error: " << strerror(s) << "\n";
		return;
	}
	if ((s = BMessenger(fFtpLooper).SendMessage(msg)) != B_OK) {
		*fLogView << "Error: " << strerror(s) << "\n";
		return;
	}
	fStage = STAGE_CHDIR;
	SetBusy(true);
}

void TFTPWindow::DirlistChanged()
{
	for(int32 i = 0; i < fRemoteFileView->CountRows(); i++) {
		BRow *row = fRemoteFileView->RowAt(i);
		BBitmapField *bitmapField = (BBitmapField *)row->GetField(CLM_ICON);
		if (bitmapField != NULL) if (bitmapField->Bitmap() != NULL) delete bitmapField->Bitmap();
	}
	fRemoteFileView->Clear();
	
	AddRemoteFileItem(".", 0, "", 01000, "", "");
	AddRemoteFileItem("..", 0, "", 01000, "", "");
	
	BString name, date, owner, group;
	uint32 size, permission;
	for(int32 i = 0; i < fDirentParser->CountEntries(); i++) {
		if (fDirentParser->GetEntryData(i, &name, &size, &date, &permission, &owner, &group) == false) continue;
		AddRemoteFileItem(name.String(), size, date.String(), permission, owner.String(), group.String());
	}
	BString status;
	status << fDirentParser->CountEntries() << " items";
	fStatusView->SetText(status.String());
	
	BRow *top = fRemoteFileView->RowAt(0);
	fRemoteFileView->ScrollTo(top);
}

void TFTPWindow::RemoteFileDoubleClicked()
{
	if (fStage != STAGE_IDLE) return;
	
	BRow *row = fRemoteFileView->CurrentSelection();
	if (row == NULL) return;
	
	BStringField *intNameField = (BStringField *)row->GetField(CLM_INTERNAL_NAME);
	BStringField *strpermField = (BStringField *)row->GetField(CLM_PERMISSION);
	
	if (strpermField->String()[0] == 'd') {
		// open directory
		this->Chdir(intNameField->String());
	} else {
		// open file
		
	}
}
