#ifndef _FTPWINDOW_H_
#define _FTPWINDOW_H_


#include <SupportDefs.h>
#include <Window.h>
#include <View.h>
#include <Button.h>
#include <TextView.h>
#include <ScrollView.h>
#include <MenuBar.h>
#include <Menu.h>
#include <MenuItem.h>
#include <TextControl.h>
#include <interface/StringView.h>
#include <Directory.h>
#include <Entry.h>
#include <Path.h>
#include <String.h>
#include <MessageRunner.h>
#include "Encoder.h"
#include "ColumnListView.h"
#include "ColumnTypes.h"
#include "FTPClient.h"
#include "ConfigFile.h"
#include "DirentParser.h"


enum {
	STAGE_SUSPENDED = 0,
	STAGE_IDLE,
	STAGE_CONNECT,
	STAGE_PASVLIST,
	STAGE_CHDIR,
};

enum {
	CLM_ICON,
	CLM_NAME,
	CLM_INTERNAL_NAME,
	CLM_SIZE,
	CLM_DATE,
	CLM_PERMISSION,
	CLM_OWNER,
	CLM_GROUP,
};

enum {
	MSG_BOOKMARK_SELECTED = 'mbse',
	MSG_SHOW_BOOKMARK = 'msbk',
	MSG_POLLING_TIMER = 'mplt',
	MSG_ENCODER_SELECTED = 'mesl',
	MSG_CANCEL = 'msca',
	MSG_ENTRY_DOUBLE_CLICKED = 'medc',
	MSG_REMOTE_PATH_CHANGED = 'rpch',
	MSG_LOG = 'mlog',
	MSG_FTP_FAILED = 'mffa',
	MSG_FTP_OK = 'mfok',
	MSG_WALKED = 'mwal',
	MSG_DISCONNECTED = 'mdic',
	
	MSG_FTP_CONNECT = 'mfcn',
	MSG_FTP_PASV_LIST = 'mfpl',
	MSG_FTP_CHDIR = 'mfch',
};

typedef struct connect_param_t {
	const char *host;
	uint16 port;
	const char *username;
	const char *password;
} connect_param_t;


typedef struct entry_t {
	BString *name;
	uint64 size;
	bigtime_t time;
	uint32 permission;
	BString *owner;
	BString *group;
} entry_t;


class TEncoderMenuItem : public BMenuItem
{
public:
	TEncoderMenuItem(const char *label, encoder_addon_t *addon)
		:	BMenuItem(label, new BMessage(MSG_ENCODER_SELECTED)) {
		fEncoderAddon = addon;
	};
	encoder_addon_t *EncoderAddon() const { return fEncoderAddon; };
private:
	encoder_addon_t *fEncoderAddon;
};

class TLogView : public BTextView
{
public:
	TLogView(BRect rect, const char *name, BRect textRect);
	~TLogView();
	void FrameResized(float width, float height);
	
	TLogView& operator<<(const char *string);
	TLogView& operator<<(int val);
	
protected:
	void InsertText(const char *text, int32 length, int32 offset, const text_run_array *runs);
};

class TFtpLooper : public BLooper
{
public:
	TFtpLooper(BLooper *owner, const connect_param_t *param);
	~TFtpLooper();
	void MessageReceived(BMessage *msg);
	void Abort();
private:
	void Connect();
	void PasvList();
	void Chdir(const char *dir);
	
	BLooper *fOwner;
	TFTPClient *fFtpClient;
	BMessenger *fMessenger;
	BMessageRunner *fPollingTimer;
	
	BString fHost;
	uint16 fPort;
	BString fUsername;
	BString fPassword;
	
	TFtpLooper& operator<<(const char *string);
	TFtpLooper& operator<<(BString string);
	TFtpLooper& operator<<(int val);
};

class TFTPWindow : public BWindow
{
public:
	TFTPWindow(BRect frame, const char *name);
	~TFTPWindow();
	
	bool QuitRequested();
	void MessageReceived(BMessage *msg);
	
private:
	uint32 fStage;
	TFtpLooper *fFtpLooper;
	TConfigFile *fBookmarkConfig;
	TDirentParser *fDirentParser;
	
	TLogView *fLogView;
	BColumnListView *fRemoteFileView;
	BTextControl *fRemotePathView;
	BStringView *fStatusView;
	BMenu *fConnectMenu;
	BMenu *fEncodingMenu;
	TEncoderMenuItem *fNoEncoder;
	
	status_t LoadBookmarks(BDirectory *dir, BMenu *menu);
	status_t AddBookmark(BEntry *entry, BMenu *menu);
	
	void AddRemoteFileItem(const char *name, uint32 size,
		const char *date, uint32 perm, const char *owner, const char *group);
	
	void SetBusy(bool busy);
	void Clear();
	
	void FtpOk();
	void FtpFailed();
	void ShowBookmark() const;
	void EncoderChanged();
	void BookmarkSelected(const char *path);
	void RemoteFileDoubleClicked();
	
	void Connect(const connect_param_t *param);
	void Disconnect();
	void PasvList();
	void Chdir(const char *dir);
	void DirlistChanged();
};

#endif
