#include <Directory.h>
#include <Entry.h>
#include <Path.h>
#include <Roster.h>
#include <Mime.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "FtpPositive.h"


TConfigFile *app_config;
BList *app_extlist;
BList *app_encoderlist;


TFtpPositive::TFtpPositive()
	:	BApplication("application/x-vnd.momoziro-ftppositive")
{
	BRect frame;
	BString v;
	
	mkdir(CONFIG_DIR, 0777);
	mkdir(BOOKMARKS_DIR, 0777);
	app_config = new TConfigFile(CONFIG_DIR "/FtpPositive.cnf");
	app_extlist = new BList();
	app_encoderlist = new BList();
	
	BMessage types, ext;
	BMimeType mime;
	BMimeType::GetInstalledTypes(&types);
	const char *typeName, *extName;
	ext_item_t *item;
	for (int32 i = 0; types.FindString("types", i, &typeName) == B_OK; i++) {
		mime.SetTo(typeName);
		if (mime.GetFileExtensions(&ext) == B_OK) {
			for(int32 k = 0; ext.FindString("extensions", k, &extName) == B_OK; k++) {
				if ((item = (ext_item_t *)malloc(sizeof(ext_item_t))) == NULL) break;
				strncpy(item->ei_ext, extName, sizeof(item->ei_ext));
				strncpy(item->ei_mimetype, typeName, sizeof(item->ei_mimetype));
				app_extlist->AddItem(item);
			}
		}
	}
	
	fLoadAddons();
	
	v.SetTo(""); app_config->Read("frame_left",   &v, "100"); frame.left   = atoi(v.String());
	v.SetTo(""); app_config->Read("frame_top",    &v, "100"); frame.top    = atoi(v.String());
	v.SetTo(""); app_config->Read("frame_right",  &v, "600"); frame.right  = atoi(v.String());
	v.SetTo(""); app_config->Read("frame_bottom", &v, "450"); frame.bottom = atoi(v.String());
	
	fFTPWindow = new TFTPWindow(frame, "FtpPositive");
	fFTPWindow->Show();
	
	/*
	//test code
	encoder_addon_t *addon = (encoder_addon_t *)fEncoderList.ItemAt(0);
	BString str;
	addon->fp_text_convert("あいうえお", &str, false);
	printf("%s\n", str.String());
	*/
	
	/*
	TFTPClient *ftp = new TFTPClient();
	status_t status = ftp->Connect("127.0.0.1", 21);
	printf("%ld %s\n", status, ftp->SocketError());
	*/
	
}

TFtpPositive::~TFtpPositive()
{
	status_t e;
	fUnloadAddons();
	if ((e = app_config->Update()) != B_OK) {
		fprintf(stderr, "%s\n", strerror(e));
	}
	delete app_config;
	
	for(int32 i = 0; i < app_extlist->CountItems(); i++) {
		free(app_extlist->ItemAt(i));
	}
	delete app_extlist;
}

void TFtpPositive::fUnloadAddons()
{
	for(int32 i = 0; i < app_encoderlist->CountItems(); i++) {
		encoder_addon_t *addon = (encoder_addon_t *)app_encoderlist->ItemAt(i);
		fprintf(stderr, "unload: %s\n", addon->name);
		unload_add_on(addon->id);
		free(addon);
	}
}

void TFtpPositive::fLoadAddons()
{
	fp_text_convert_t fp_text_convert;
	image_id id;
	BDirectory dir;
	status_t e;
	app_info info;
	if ((e = GetAppInfo(&info)) != B_OK) {
		fprintf(stderr, "fLoadAddons: GetAppInfo() failed. %s", strerror(e));
		return;
	}
	BEntry entry(&info.ref), parent;
	
	if ((e = entry.GetParent(&parent)) != B_OK) {
		fprintf(stderr, "fLoadAddons: GetParent() failed. %s", strerror(e));
		return;
	}
	BPath path(&parent);
	path.Append("add-ons");
	
	// load encoders add-on
	BPath addonPath(path);
	addonPath.Append("encoders");
	dir.SetTo(addonPath.Path());
	while (dir.GetNextEntry(&entry) == B_OK) {
		BPath addonPath(&entry);
		if ((id = load_add_on(addonPath.Path())) < 0) continue;
		if (get_image_symbol(id, "fp_text_convert", B_SYMBOL_TYPE_TEXT, (void **)&fp_text_convert)) {
			unload_add_on(id);
			continue;
		}
		encoder_addon_t *addon = (encoder_addon_t *)malloc(sizeof(encoder_addon_t));
		if (!addon) {
			fprintf(stderr, "%s\n", strerror(ENOMEM));
			unload_add_on(id);
			return;
		}
		addon->id = id;
		addon->fp_text_convert = fp_text_convert;
		strncpy(addon->name, addonPath.Leaf(), sizeof(addon->name));
		app_encoderlist->AddItem(addon);
		fprintf(stderr, "load: %s\n", addon->name);
	}
	
}
