#include <Bitmap.h>
#include <Mime.h>
#include <stdio.h>
#include <stdlib.h>
#include "FtpPositive.h"
#include "RemoteFileView.h"


TRemoteFileItem::TRemoteFileItem(const char *name, const char *size,
		const char *timestamp, const char *permission,
		const char *owner, const char *group,
		bool isDirectory)
	:	CLVEasyItem(0, false, false, 20)
{
	BMimeType mime;
	BBitmap *icon = new BBitmap(BRect(0, 0, B_MINI_ICON - 1, B_MINI_ICON - 1), B_CMAP8);
	if (isDirectory) {
		mime.SetTo("application/x-vnd.Be-directory");
	} else {
		mime.SetTo("application/octet-stream");
		const char *ext = strrchr(name, '.');
		if (ext) {
			ext++;
			ext_item_t *item;
			for (int32 i = 0; i < app_extlist->CountItems(); i++) {
				item = (ext_item_t *)app_extlist->ItemAt(i);				
				if (strcasecmp(ext, item->ei_ext) == 0) {
					mime.SetTo(item->ei_mimetype);
					break;
				}
			}
		}
	}
	if (mime.GetIcon(icon, B_MINI_ICON) != B_OK) {
		mime.SetTo("application/octet-stream");
		mime.GetIcon(icon, B_MINI_ICON);
	}
	
	SetColumnContent(RFV_ICON, icon, 2, false);
	SetColumnContent(RFV_NAME, name);
	SetColumnContent(RFV_SIZE, size, true, true);
	SetColumnContent(RFV_TIMESTAMP, timestamp);
	SetColumnContent(RFV_PERMISSION, permission);
	SetColumnContent(RFV_OWNER, owner);
	SetColumnContent(RFV_GROUP, group);
}

TRemoteFileItem::~TRemoteFileItem()
{
}

TRemoteFileView::TRemoteFileView(BRect frame, const char *name)
	:	ColumnListView(frame, &iContainerView, name,
						B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE,
						B_MULTIPLE_SELECTION_LIST,	// view type
						false,					// hierarchical
						true,					// horizontal scrollbars
						true,					// vertical scrollbars
						true,					// scroll-view corner
						B_FANCY_BORDER)			// border style
{
	this->AddColumn(new CLVColumn(NULL, 20, CLV_LOCK_AT_BEGINNING | CLV_NOT_MOVABLE | CLV_NOT_RESIZABLE | CLV_PUSH_PASS | CLV_MERGE_WITH_RIGHT));
	this->AddColumn(new CLVColumn("Name", 150, CLV_SORT_KEYABLE | CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50) );
	this->AddColumn(new CLVColumn("Size", 70, CLV_SORT_KEYABLE | CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50) );
	this->AddColumn(new CLVColumn("Timestamp", 90, CLV_SORT_KEYABLE | CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50) );
	this->AddColumn(new CLVColumn("Permission", 80, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50) );
	this->AddColumn(new CLVColumn("Owner", 70, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50) );
	this->AddColumn(new CLVColumn("Group", 70, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50) );
}

TRemoteFileView::~TRemoteFileView()
{
}

void TRemoteFileView::MessageReceived(BMessage *msg)
{
}
