#include <OS.h>
#include <string.h>
#include "TCPStream.h"


TTCPStream::TTCPStream(BDataIO *dataIO, struct sockaddr_in *addr, bigtime_t waitTime)
{
	fDataIO = dataIO;
	fWaitTime = waitTime;
	fEndpoint = socket(AF_INET, SOCK_STREAM, 0);
	if (fEndpoint == -1) {
		fStatus = errno;
		return;
	}
	
	// 非ブロッキングモード
	int block = 1;
	if (setsockopt(fEndpoint, SOL_SOCKET, SO_NONBLOCK, &block, sizeof(block)) < 0) {
		fStatus = errno;
		return;
	}
	
	// 接続
	if (connect(fEndpoint, (struct sockaddr *)addr, sizeof(struct sockaddr_in)) != 0) {
		fStatus = errno;
		if ((fStatus != 0) && (fStatus != EISCONN) && (fStatus != EINPROGRESS) && (fStatus != EALREADY)) {
			return;
		}
	}
	
	fAborted = false;
	
	// 受信スレッド開始
	fThreadID = spawn_thread(TTCPStream::ReceiveFunc, "TCPStream", B_NORMAL_PRIORITY, this);
	if (fThreadID < 0) {
		fStatus = fThreadID;
		return;
	}
	if ((fStatus = resume_thread(fThreadID)) != B_OK) return;
	
	fStatus = EINPROGRESS;
}

TTCPStream::~TTCPStream()
{
	if (fEndpoint != -1) close(fEndpoint);
}

bool TTCPStream::IsReceivable(long sec, long msec)
{
	fd_set fdset;
	int re;
	struct timeval timeout;
	
	FD_ZERO(&fdset);
	FD_SET(fEndpoint, &fdset);
	
	timeout.tv_sec = sec;
	timeout.tv_usec = msec;
	
	re = select(fEndpoint + 1, &fdset, NULL, NULL, &timeout);
	
	return (re == 1) ? true : false;
}

int32 TTCPStream::ReceiveFunc(void *self)
{
	char buff[1024];
	int32 blen;
	TTCPStream *Self = (TTCPStream *)self;
	bigtime_t limit = system_time() + Self->fWaitTime;
	while (!Self->fAborted) {
		if (Self->IsReceivable(0, 100)) {
			blen = read(Self->fEndpoint, buff, sizeof(buff));
			if (blen <= 0) {
				// Disconnected
				Self->fStatus = B_OK;
				break;
			} else {
				// Received
				limit = system_time() + Self->fWaitTime;
				Self->fDataIO->Write(buff, blen);
			}
		} else {
			// No data
			if (system_time() > limit) {
				// Timeout!
				Self->fStatus = B_TIMED_OUT;
				break;
			}
		}
	}
	return 0;
}

void TTCPStream::Abort()
{
	status_t exit_status;
	fAborted = true;
	wait_for_thread(fThreadID, &exit_status);
}

